REM***********************************************************************
REM                                                                      *
REM --- SAILING in QBASIC --------------- by Stephan Scholz, March 2012  *
REM     -----------------                 ----------------------------   *
REM                                                                      *
REM Sail round a three-buoy course in a diamond-shaped sailing yacht and *
REM try to beat the AI opponent. You control sail, rudder and spinnaker. *
REM                                                                      *
REM LEFT-RIGHT CURSOR ARROWS: Tap these for left-right rudder.           *
REM    UP-DOWN CURSOR ARROWS: Tap these to let out or pull in the boom.  *
REM                    ENTER: Press to deploy or stow SPINNAKER.         *
REM                                                                      *
REM Depending on your heading in relation to the wind, and also on the   *
REM boom position, you will be able to sail your yacht.                  *
REM                                                                      *
REM A wind instrument at the top of the screen shows the wind direction  *
REM and your speed. The white line indicates your relative or true wind, *
REM and the blue line shows your apparent wind, which varies with speed. *
REM                                                                      *
REM The colour of your sail boom will also give information:             *
REM                                                                      *
REM LIGHT BLUE BOOM: Sail in laminar flow, with effective 12-25 angles  *
REM of incidence of wind on sail. 40% force comes from the pressure on   *
REM the rear surface, and 60% from the vaccuum on the front              *
REM                                                                      *
REM ORANGE BOOM: Non-laminar flow. The wind only pushes the sail.        *
REM                                                                      *
REM GREY BOOM: Sail flapping loosely in the wind - no push at all! The   *
REM yacht will only turn slowly as if you were using an oar!             *                                      
REM                                                                      *
REM Let out boom for rear winds and pull in to sail with forward wind.   *
REM REMEMBER: The wind comes from the north, and you cant sail directly *
REM against it! For this, you will have to tack.                         *
REM                                                                      *
REM The SPINNAKER can only be used when the wind comes from behind, upto *
REM perpendicularly from the side.                                       *                    
REM                                                                      *
REM When the race starts, head for the area circled in red near a given  *
REM buoy. Once you enter, the next buoy on the course will have its area *
REM circled in red. For the moment, there is no lap-counter, and the     *
REM never stops. The small blue circles are waypoints for the AI yacht.  *
REM                                                                      *
REM Propulsion and apparent wind speeds and angles are calculated using  *
REM formulae quoted in sailing manuals. Sail push depends on apparent    *
REM wind angle of incidence on the sail, which defines percentage force  *
REM to be used for the propulsion vector of the boom angle at a given    *
REM ship's heading. Air and water resistances are also accounted for.    *
REM                                                                      *
REM There is a 23-knot north wind.                                       *
REM                                                                      *
REM Speed calibration closest to a fast racing yacht in a 23 Kt wind:    *
REM 40 degrees, close hauled: 12 kt.                                     *
REM 45 degrees:               14 kt.                                     *
REM 90 degrees, reaching:     16 kt.                                     *
REM 135 degrees, running:     10 kt,   20  kt with Spinnaker             *
REM 180 degrees, running:    6.6 kt,   9.4 kt with Spinnaker             *
REM                                                                      *
REM An AI sailing yacht tries to beat you. Collision detection enabled!  *
REM                                                                      *
REM More detailed on-screen information on wind speed and angles can be  *
REM toggled by pressing "i" (for info).                                  *
REM                                                                      *
REM PRESS "q" TO END     NOTE: Caps Lock must be disabled.               *                      
REM                                                                      *
REM***********************************************************************

SCREEN 12: CLS
REM ------------------ screen size
h = 480
w = 640
REM --------------- starting position
x = 600
y = 420
ve = 0
hullcol = 2
hull = 18: REM -------- hull size
boom = hull / 1.3
rudder = hull / 3.5
mastpos = hull / 2.2
heading = 10

boomangle = 35
flecha = 17: REM ------ body width
rudderangle = 45
transom = hull / 2.7
transomangle = 130

REM ---------------- Competitor, initial variable values
xu = 620
yu = 400
veu = 0
hullcolu = 6
headingu = 50
boomangleu = 35
rudderangleu = 0
spi = -1

REM ---------------------- some constants

windspeed = 23
windangle = -90
pi = 3.14162
max = 60

REM ---------------------- on screen speed of movement
speedscale = .08
turnscale = .125
                              
REM ---------------------- wind instrument position
wix = 100
wiy = 64

DIM se(180)
DIM spin(180)
DIM cd(360)
DIM rsf(180)

near = 30
buoyzone = 30

GOSUB Sailefficiency
GOSUB Spinaker
GOSUB resfactor
GOSUB Competitordata

REM ------------------- Set waypoints
DIM waypx(4)
DIM waypy(4)
DIM waypxu(7)
DIM waypyu(7)
DIM boya(7): REM turning sharpness.
GOSUB waypoint
wp = 1
wpu = 1

REM +++++++++++++++++++++++++++++ MAIN LOOP +++++++++++++++++++++++++++++++++
REM -------------------------------------------------------------------------

DO

REM -------------- DETECT BUOY -----------------------

FOR n = 1 TO 4: CIRCLE (waypx(n), waypy(n)), buoyzone, 1: NEXT
CIRCLE (waypx(wp), waypy(wp)), buoyzone, 12

hdis = INT((x - waypx(wp)))
vdis = INT((y - waypy(wp)))
IF vdis <> 0 THEN prop = hdis / vdis
atan = ATN(prop) / pi * 180

REM -- Depending on the trigonometrical quadrant of your position in
REM -- relation to the buoy you must head for, the arctangent of the
REM -- horizontal distance divided by the vertical distance will give
REM -- you the bearing angle to the buoy.

IF SGN(hdis) = -1 AND SGN(vdis) = 1 THEN bearing = -atan
IF SGN(hdis) = 1 AND SGN(vdis) = 1 THEN bearing = 360 - atan
IF SGN(hdis) = 1 AND SGN(vdis) = -1 THEN bearing = 180 - atan
IF SGN(hdis) = -1 AND SGN(vdis) = -1 THEN bearing = 180 - atan

IF ABS(hdis) < 21 AND ABS(vdis) < 21 THEN
       SOUND 300, .3: SOUND 600, .3: SOUND 900, .3
       CIRCLE (waypx(wp), waypy(wp)), buoyzone, 7
       wp = wp + 1
       IF wp > 4 THEN wp = 1
END IF

headi = heading - 90
IF headi < 0 THEN headi = headi + 360
IF headi > 360 THEN headi = headi - 360

REM ------------------------ RELATIVE WIND ----------------------------------
REM ------ Wind angle in relation to your heading.

relwind = (windangle - heading) + 180
IF relwind < -180 THEN relwind = relwind + 360
IF relwind > 360 THEN relwind = relwind - 360

REM ---------------- WINDWARD SIDE RIGHT(+) OR LEFT(-) ----------------------

tk = SGN(relwind)

REM ------------------- VELOCITY OF APPARENT WIND -----------------------
REM ---------------------- ANGLE OF APPARENT WIND -----------------------
REM ----- Apparent wind angle in relation to your boat - varies with speed.

Vay = windspeed * SIN(pi * relwind / 180)
Vax = windspeed * COS(pi * relwind / 180) + ve
Vapwind = SQR(Vax * Vax + Vay * Vay)
Aapwind = (180 / pi) * ATN(Vay / Vax)
IF ABS(relwind - Aapwind) > 179 THEN Aapwind = Aapwind + 180 * tk

REM --------------------------- BOOM ANGLE ----------------------------------
REM ----- Boom colour changes with wind angle of incidence:
REM ----- laminar flow, push, flapping or too close hauled

sil = boomangle * tk
sail = sil
difang = ABS(Aapwind) - ABS(sil)
IF difang > 11 AND difang < 26 THEN boomcol = 11 ELSE boomcol = 12

REM ------------- boom will trail if loose in the wind
IF ABS(sil) >= ABS(Aapwind) THEN sail = Aapwind: difang = 0: boomcol = 8

REM ------------------------ WIND FORCE VECTORS ----------------------------
REM -- Sail efficiency depends on apparent wind angle of incidence
REM -- on sail, and will determine percentage of apparent wind
REM -- force to be used for propulsion vector depending on boomangle.

ins = INT(difang)
sefi = se(ins)
fsail = ABS(sil * pi / 180)

REM ----------------------WATER AND AIR RESISTANCE -------------------------
REM --- Resistance factor is read from the resistance table for each sailing
REM --- angle and multiplied to the square of the yacht's speed.

rsf = rsf(ABS(relwind))
res = ve * ve * rsf

REM --------------------- FINAL PROPULSION CALCULATION ----------------------
REM --- For raised spinnaker this force is read from the spinnaker table for
REM --- each sailing angle and added.
REM --- Resistance is subtracted.

IF spi = 1 THEN spina = spin(ABS(relwind)) * 1.5 ELSE spina = 0
prop = .0246 * sefi * SIN(fsail) * Vapwind - res + spina

REM ------------- ACCELLERATE/DECELLERATE AS PER PROPULSION ----------------

ve = ve - SGN(ve - prop) * .05
IF ve > max THEN ve = max
IF ve < 0 THEN ve = 0

REM ======================= DRAW SAILING YACHT =============================
REM --------- hull line angles, boom and rudder positions and angles -------

n = (heading + flecha) * pi / 180
n1 = (heading - flecha) * pi / 180
n2 = heading * pi / 180
boomang = heading + sail
n3 = boomang * pi / 180
rudderang = heading + rudderangle
n4 = rudderang * pi / 180
n5 = (heading + transomangle) * pi / 180
n6 = (heading - transomangle) * pi / 180

REM --------------------- ERASE HULL, BOOM AND RUDDER ----------------------
WAIT &H3DA, 8
FOR p = 1 TO 10000: NEXT p

LINE (x, y)-(x1, y1), 0
LINE (x, y)-(x2, y2), 0
LINE (x5, y5)-(x6, y6), 0
LINE (x5, y5)-(x7, y7), 0
LINE (x5, y5)-(x8, y8), 0
CIRCLE (x3, y3), 2, 0, , , 1
LINE (x3, y3)-(x4, y4), 0

REM ------------- ERASE COMPETITOR HULL, BOOM AND RUDDER -------------------

LINE (xu, yu)-(x1u, y1u), 0
LINE (xu, yu)-(x2u, y2u), 0
LINE (x5u, y5u)-(x6u, y6u), 0
LINE (x5u, y5u)-(x7u, y7u), 0
LINE (x5u, y5u)-(x8u, y8u), 0
LINE (x3u, y3u)-(x4u, y4u), 0

REM ----------------------- ERASE WIND -------------------------------------

LINE (wix, wiy)-(xw, yw), 0
LINE (wix, wiy)-(xaw, yaw), 0


REM ------------------- KEEP IT ALL ON SCREEN ------------------------------

IF x > w THEN x = w
IF y > h THEN y = h
IF x < 0 THEN x = 0
IF y < 0 THEN y = 0

REM ----------- SPEED VECTORS FOR ON-SCREEN MOVEMENT -----------------------

x = x - (ve * COS(n2)) * speedscale
y = y - (ve * SIN(n2)) * speedscale

REM --------- PIXEL COORDS AND ANGLE FOR HULL, BOOM AND RUDDER -------------

x1 = x + hull * COS(n)
y1 = y + hull * SIN(n)
x2 = x + hull * COS(n1)
y2 = y + hull * SIN(n1)
x3 = x + mastpos * COS(n2)
y3 = y + mastpos * SIN(n2)
x4 = x3 + boom * COS(n3)
y4 = y3 + boom * SIN(n3)
x5 = x + hull * 1.2 * COS(n2)
y5 = y + hull * 1.2 * SIN(n2)
x6 = x5 + rudder * COS(n4)
y6 = y5 + rudder * SIN(n4)
x7 = x5 + transom * COS(n5)
y7 = y5 + transom * SIN(n5)
x8 = x5 + transom * COS(n6)
y8 = y5 + transom * SIN(n6)

REM ---------------- WIND POSITION ON INSTRUMENT ---------------------------

wind = (Aapwind - 90) * pi / 180
rwind = (relwind - 90) * pi / 180
xw = wix + windspeed * 2 * COS(rwind)
yw = wiy + windspeed * 2 * SIN(rwind)
xaw = wix + windspeed * 2 * COS(wind)
yaw = wiy + windspeed * 2 * SIN(wind)

REM ------------------ DRAW HULL, BOOM AND RUDDER --------------------------

LINE (x, y)-(x1, y1), hullcol
LINE (x, y)-(x2, y2), hullcol
LINE (x5, y5)-(x6, y6), 14
LINE (x5, y5)-(x7, y7), hullcol
LINE (x5, y5)-(x8, y8), hullcol
LINE (x3, y3)-(x4, y4), boomcol


REM ======================= AUTOMATIC YACHT =================================
REM =========================================================================

headu = headingu - 90
IF headu < 0 THEN headu = headu + 360

REM -------------- DETECT BUOY ----------------------------------------------

REM CIRCLE (waypxu(wpu), waypyu(wpu)), buoyzone, 12
IF ABS(xu - waypxu(wpu)) < 21 AND ABS(yu - waypyu(wpu)) < 21 THEN
       SOUND 200, .3: SOUND 400, .3: SOUND 600, .3
       REM CIRCLE (waypxu(wpu), waypyu(wpu)), buoyzone, 7
      
       wpu = wpu + 1: IF wpu > 7 THEN wpu = 1
       check = 0: look = 0
END IF

REM ----------------- DETERMINE HEADING --------------------------------------

REM -- When AI Yacht arrives at a buoy or waypoint, the turning
REM -- sharpness to get round and along the next leg is read
REM -- from a table and set.

buoyu = 12 * boya(wpu)
hdisu = xu - waypxu(wpu)
vdisu = yu - waypyu(wpu)

IF vdisu <> 0 THEN propu = hdisu / vdisu
atanu = ATN(propu) / pi * 180

REM -------------WAYPOINT BEARING CALCULATION ---------------------------
REM --- Depending on the trigonometrical quadrant of AI yacht's position
REM --- in relation to the buoy or waypoint it must head for, the
REM --- arctangent of the horizontal distance divided by the vertical
REM --- distance will give the bearing angle to follow. The angle is
REM --- calculated at certain time intervals to allow time for tacking.

check = check - 1
IF check < 1 THEN
  check = 45
  IF SGN(hdisu) = -1 AND SGN(vdisu) = 1 THEN bearingu = -atanu
  IF SGN(hdisu) = 1 AND SGN(vdisu) = 1 THEN bearingu = 360 - atanu
  IF SGN(hdisu) = 1 AND SGN(vdisu) = -1 THEN bearingu = 180 - atanu
  IF SGN(hdisu) = -1 AND SGN(vdisu) = -1 THEN bearingu = 180 - atanu
END IF

difu = ABS(headu - bearingu)
IF difu < 12 AND ABS(relwindu) > 40 THEN GOTO fiu

REM -------- Turn direction, depending on the difference between
REM -------- current heading and bearing, allowing time for tacking.

look = look - 1
IF look < 1 THEN
       IF bearingu < headu THEN
          look = 30
          IF difu > 180 THEN rudderangleu = -buoyu ELSE rudderangleu = buoyu
       END IF

       IF bearingu > headu THEN
          look = 25
          IF difu > 180 THEN rudderangleu = buoyu ELSE rudderangleu = -buoyu
       END IF
END IF
fiu:

REM -------------- HERE YOU CAN ADJUST AUTOMATIC YACHT SPEED ---------------
REM -------------------- by changing the value 120 -------------------------
veul = cd(headu) * windspeed / 120
IF veul < veu THEN veu = veu - speedscale
IF veul > veu THEN veu = veu + speedscale

REM ----------------- AUTOMATIC YACHT BOOM ANGLE AI ------------------------

relwindu = (windangle - headingu) + 180
IF relwindu < -180 THEN relwindu = relwindu + 360
IF relwindu > 360 THEN relwindu = relwindu - 360
tku = SGN(relwindu)
silu = boomangleu * tku
sailu = silu
difangu = ABS(relwindu) - ABS(silu)
IF difangu > 9 AND difangu < 26 THEN boomcolu = 11 ELSE boomcolu = 12

REM -------------- BOOM LOOSE IN THE WIND --------------------------

IF ABS(silu) > ABS(relwindu) THEN sailu = relwindu: difangu = 0: boomcolu = 8
IF boomcolu = 12 AND headingu > 180 AND boomangleu < 75 THEN boomangleu = boomangleu + 1
IF boomcolu = 12 AND headingu < 180 AND boomangleu > 20 THEN boomangleu = boomangleu - 1


REM -------------------- AI YACHT AUTOMATIC TURNING ----------------

IF rudderangleu < 0 THEN headingu = headingu + .1 + (turnscale / 2 * veu): rudderangleu = rudderangleu + .5
IF rudderangleu > 0 THEN headingu = headingu - .1 - (turnscale / 2 * veu): rudderangleu = rudderangleu - .5

IF headingu < 1 THEN headingu = 360
IF headingu > 360 THEN headingu = 1

REM -------- HULL LINE ANGLES, BOOM AND RUDDER POSITIONS AND ANGLES --------

nu = (headingu + flecha) * pi / 180
n1u = (headingu - flecha) * pi / 180
n2u = headingu * pi / 180
boomangu = headingu + sailu
n3u = boomangu * pi / 180
rudderangu = headingu + rudderangleu
n4u = rudderangu * pi / 180
n5u = (headingu + transomangle) * pi / 180
n6u = (headingu - transomangle) * pi / 180

REM ------------------- KEEP IT ON SCREEN ----------------------------------

IF xu > w THEN xu = w
IF yu > h THEN yu = h
IF xu < 0 THEN xu = 0
IF yu < 0 THEN yu = 0

REM -------------------- CALCULATE SPEED VECTORS ---------------------------

xu = xu - (veu * COS(n2u)) * speedscale
yu = yu - (veu * SIN(n2u)) * speedscale

REM ------ PIXEL COORDS AND ANGLE FOR HULL, BOOM AND RUDDER -----------------

x1u = xu + hull * COS(nu)
y1u = yu + hull * SIN(nu)
x2u = xu + hull * COS(n1u)
y2u = yu + hull * SIN(n1u)
x3u = xu + mastpos * COS(n2u)
y3u = yu + mastpos * SIN(n2u)
x4u = x3u + boom * COS(n3u)
y4u = y3u + boom * SIN(n3u)
x5u = xu + hull * 1.2 * COS(n2u)
y5u = yu + hull * 1.2 * SIN(n2u)
x6u = x5u + rudder * COS(n4u)
y6u = y5u + rudder * SIN(n4u)
x7u = x5u + transom * COS(n5u)
y7u = y5u + transom * SIN(n5u)
x8u = x5u + transom * COS(n6u)
y8u = y5u + transom * SIN(n6u)

REM ------------- DRAW HULL, BOOM AND RUDDER -----------------

LINE (xu, yu)-(x1u, y1u), hullcolu
LINE (xu, yu)-(x2u, y2u), hullcolu
LINE (x5u, y5u)-(x6u, y6u), 14
LINE (x5u, y5u)-(x7u, y7u), hullcolu
LINE (x5u, y5u)-(x8u, y8u), hullcolu
LINE (x3u, y3u)-(x4u, y4u), boomcolu

REM ================== DRAW BUOYS ===========================================

CIRCLE (bx1, by1), 4, 10, , , 1
PAINT (bx1, by1), 10, 10
CIRCLE (bx2, by2), 4, 14, , , 1
PAINT (bx2, by2), 14, 14
CIRCLE (bx3, by3), 4, 9, , , 1
PAINT (bx3, by3), 9, 9
CIRCLE (bx4, by4), 4, 12, , , 1
PAINT (bx4, by4), 12, 12

CIRCLE (waypxu(2), waypyu(2)), 4, 9, , , 1
CIRCLE (waypxu(3), waypyu(3)), 4, 9, , , 1
CIRCLE (waypxu(6), waypyu(6)), 4, 9, , , 1

i$ = INKEY$
IF i$ = "" THEN i$ = " "
IF i$ = "q" THEN END

REM --------------------------- CONTROL KEYS ------------------------------

nt% = ASC(RIGHT$(i$, 1))
SELECT CASE nt%
REM keys "o" and "p" can steer competitor
CASE 111
IF rudderangleu < 45 THEN rudderangleu = rudderangleu + 5
CASE 112
IF rudderangleu > -45 THEN rudderangleu = rudderangleu - 5
CASE 80
IF boomangle > 0 THEN boomangle = boomangle - 1
CASE 72
IF boomangle < 90 THEN boomangle = boomangle + 1
CASE 75
IF rudderangle < 45 THEN rudderangle = rudderangle + 5
CASE 77
IF rudderangle > -45 THEN rudderangle = rudderangle - 5
CASE 73
IF windspeed < 30 THEN windspeed = windspeed + 1
CASE 81
IF windspeed > 0 THEN windspeed = windspeed - 1
CASE 13: REM ENTER deploys or stows spinnaker.
BEEP
spi = -spi

CASE 105:
IF info = 0 THEN
  info = 1
ELSE
  CLS : info = 0
END IF

END SELECT

REM ----------------------- RUDDER EFFECTIVITY ---------------------------
REM ---- Depends on speed and includes return back to neutral.
REM ---- Zero speed turns are possible - as if using an oar.

IF rudderangle < 0 THEN heading = heading + .2 + (turnscale * ve): rudderangle = rudderangle + 2.5
IF rudderangle > 0 THEN heading = heading - .2 - (turnscale * ve): rudderangle = rudderangle - 2.5
IF heading < 0 THEN heading = heading + 360
IF heading > 360 THEN heading = heading - 360

REM ----------------------- COLLISION DETECTION --------------------------

detx = INT(ABS(x3 - x3u))
dety = INT(ABS(y3 - y3u))
IF detx < 12 AND dety < 12 THEN SOUND 200, .1

REM ----------------------- ON-SCREEN INFORMATION ------------------------
REM---------- Draw wind instrument with true and apparent wind angles ----

velo = INT(ve * 10) / 10

COLOR 14:
LOCATE 5, wix / 9: PRINT velo; "  "
LOCATE 6, 1 + wix / 9: PRINT "Spi ";
IF spi = 1 THEN PRINT CHR$(24)
IF spi = -1 THEN PRINT CHR$(25)

CIRCLE (wix, wiy), 50, 2, , , 1

LINE (wix, wiy)-(xw, yw), 15
LINE (wix, wiy)-(xaw, yaw), 11
LINE (wix - 15, wiy)-(wix + 21, wiy + 13), 7, B

COLOR 2: LOCATE 1, wix / 5: PRINT "Heading:"; INT(headi)
COLOR 12: LOCATE 2, wix / 5: PRINT "Bearing:"; INT(bearing)

IF info = 1 THEN
  REM -------------- Info on your ship
  COLOR 7
  LOCATE 3, wix / 5: PRINT " rel wind:"; INT(relwind * 10) / 10; "  "
  LOCATE 4, wix / 5: PRINT "v app wind :"; INT(Vapwind * 10) / 10; "kt. "
  LOCATE 5, wix / 5: PRINT " app wind :"; INT(Aapwind * 10) / 10; "  "
  LOCATE 6, wix / 5: PRINT "waypoint   :"; wp; " "
  REM -------------- Info on competitor
  COLOR 6
  LOCATE 1, wix / 1.7: PRINT "heading :"; INT(headu); "  "
  LOCATE 2, wix / 1.7: PRINT "bearing :"; INT(bearingu); "  "
  LOCATE 3, wix / 1.7: PRINT "rel wind:"; INT(relwindu); "  "
  LOCATE 4, wix / 1.7: PRINT "speed   :"; INT(veu * 10) / 10; "kt "
  LOCATE 5, wix / 1.7: PRINT "waypoint:"; wpu;
  LOCATE 6, wix / 1.7: PRINT " turns :"; buoyu; "  "
END IF



LOOP

REM --------------------- SAIL EFFICIENCY DATA ----------------------------

Sailefficiency:
RESTORE incidence
FOR n = 1 TO 180
READ A
se(n) = A
NEXT n
incidence:
DATA 0,0,0,0,0,0,0,0,0,0
DATA 80,81,83,85,87,89,91,93,95,97
DATA 98,99,100,99,98,97,70,70,70,70
DATA 70,70,70,70,70,70,70,70,70,70
DATA 70,70,70,70,70,70,71,71,72,72
DATA 73,73,74,74,75,75,75,75,75,75
DATA 75,75,75,75,75,75,75,75,75,75
DATA 75,75,75,75,75,75,75,75,75,75
DATA 75,76,77,78,79,80,80,80,80,80
DATA 79,78,77,76,75,74,73,72,71,70
DATA 69,68,67,66,65,64,63,62,61,60
DATA 59,58,57,56,55,54,53,52,51,50
DATA 49,48,47,46,45,44,43,42,41,40
DATA 39,38,37,36,35,34,33,32,31,30
DATA 29,28,27,26,25,24,23,22,21,20
DATA 19,18,17,16,15,14,13,12,11,10
DATA 9,8,7,6,5,4,3,2,1,0
DATA 0,0,0,0,0,0,0,0,0,0,0
RETURN

REM -------------------- SPINNAKER DATA ----------------------

Spinaker:
REM useless angles:
FOR n = 1 TO 89
spin(n) = -100
NEXT n
REM useful angles:
RESTORE spi
FOR n = 90 TO 180
READ A
spin(n) = A
NEXT n
spi:
DATA 1,2,3,4,5,6,7,8,9,10
DATA 11,12,13,14,15,16,17,18,19,20
DATA 22,24,26,28,30,32,34,36,38,40
DATA 42,44,46,48,50,52,54,56,58,60
DATA 62,65,68,71,74,75,78,81,83,85
DATA 88,87,86,85,84,83,82,81,80,79
DATA 78,76,74,72,70,68,66,64,62,60
DATA 58,56,54,52,50,48,46,44,42,40
DATA 38,36,34,32,30,28,26,25,24,23,23

RETURN

REM ------------------- RESISTANCE FACTOR DATA --------------

REM ---- Wind resistance increases for close-hauled angles --------
REM ---- Headway below 40 degrees relative wind is nil ------------
REM ---- Water resistance increases for running angles due to pitching

REM -- Closest calibration to a racing yacht in a 23 kt wind:
REM -- close-hauled 40 deg: 12 kt
REM -- close-hauled 45 deg: 14 kt
REM -- reaching  90 deg:    16 kt
REM -- reaching 135 deg:    10 kt     20   with spi
REM -- running  180 deg:      6.6 kt   9.4 with spi

REM -- For different relative wind angles, values are read
REM -- from the table and multiplied by the value of the angle
REM -- in degrees, giving the resistance factor for that angle.

resfactor:

RESTORE Resistance
FOR n = 1 TO 180
READ A
rsf(n) = A
NEXT n
Resistance:
DATA 500,500,500,500,500,500,500,500,500,500
DATA 500,500,500,500,500,500,500,500,500,500
DATA 500,500,450,400,350,300,250,200,120,118
DATA 114,110,106,103,100,97,94,91,88,84
DATA 80,75,70,65,60,61,62,63,64,65
DATA 66,67,68,69,70,71,72,73,74,75
DATA 76,77,78,79,80,81,82,83,84,84
DATA 84,84,84,84,84,84,84,84,84,84
DATA 84,84,84,84,84,84,84,84,84,84

DATA 84,84,86,88,90,92,94,96,98,100
DATA 102,104,108,110,112,114,116,118,120,122
DATA 121,124,127,130,133,136,140,145,150,155
DATA 160,165,170,175,180,185,190,195,200,205

DATA 210,215,220,225,230,238,246,254,262,270
DATA 270,270,270,270,270,275,280,285,290,295
DATA 300,305,310,315,315,315,315,315,315,315
DATA 320,320,320,320,320,320,320,320,320,320
DATA 325,325,325,325,325,325,325,325,325,325

FOR n = 1 TO 180
rsf(n) = rsf(n) / 100000 * n
NEXT n

RETURN


REM ------------- AUTOMATIC YACHT PERFORMANCE DATA ------------------------

Competitordata:
RESTORE autos
FOR n = 30 TO 180
READ A
cd(n) = A
cd(360 - n) = A
NEXT n
autos:
DATA 22,23,24,25,26,27,28,29,30,31
DATA 32,36,40,44,46,50,53,33,38,60
DATA 62,64,66,68,70,71,72,73,74,75
DATA 76,77,78,79,80,80,81,81,82,82
DATA 83,83,84,84,85,85,86,87,88,89
DATA 90,91,92,93,94,95,96,97,98,99
DATA 100,100,100,100,100,100,100,100,100,100
DATA 99,99,98,98,97,97,96,96,95,95
DATA 94,94,93,93,92,92,91,91,90,90
DATA 89,89,89,88,88,88,87,87,87,86
DATA 86,86,85,85,85,84,84,84,83,83
DATA 83,82,82,82,81,81,81,80,80,79
DATA 79,78,77,76,75,74,73,72,71,70
DATA 69,68,67,66,65,64,63,62,61,60
DATA 59,58,57,56,55,54,53,52,51,50,50
RETURN

waypoint:

REM ---------- BUOY POSITIONS -------------------------

bx1 = 540
by1 = 390
bx2 = 420
by2 = 90
bx3 = 80
by3 = 370
bx4 = 500
by4 = 415

REM -------- waypoint positions adjusted at points near buoys
REM
REM -------- For your yacht

waypx(1) = bx1 + near
waypy(1) = by1
waypx(2) = bx2
waypy(2) = by2 - near
waypx(3) = bx3 - near + 8
waypy(3) = by3 + near - 8
waypx(4) = bx4
waypy(4) = by4 + near

REM ------- Buoy/waypoints for automatic competitor. Numbers
REM ------- 2,3 and 6 are only secondary waypoints to set
REM ------- the route so AI yacht can tack with enough speed.

waypxu(1) = bx1 + near
waypyu(1) = by1

waypxu(2) = bx2
waypyu(2) = by2 + 200
waypxu(3) = bx2 + 100
waypyu(3) = by2 + 75

waypxu(4) = bx2
waypyu(4) = by2 - near
waypxu(5) = bx3 - near + 8
waypyu(5) = by3 + near - 8
waypxu(6) = bx3 + 100
waypyu(6) = by4 + 50
waypxu(7) = bx4
waypyu(7) = by4 + near

REM ------- Sets the turning speed at this buoy/waypoint

boya(1) = 1
boya(2) = 4
boya(3) = 2
boya(4) = 5
boya(5) = 1
boya(6) = 5
boya(7) = 1

RETURN

